@@ -1,11 +1,9 @@
 import click
 
-
 backend_choices = click.Choice(
     [
+        'encrypted-file',
         'file',
-        'encrypted-file'
     ],
-    case_sensitive=False
+    case_sensitive=False,
 )
-
