@@ -2,43 +2,42 @@ import click
 
 output_format_choices = click.Choice(
     [
-        'json',
-        'yaml',
         'csv',
-        'table',
+        'json',
         'list',
-        'table-plain',
-        'table-simple',
+        'table',
+        'table-double_grid',
+        'table-double_outline',
+        'table-fancy_grid',
+        'table-fancy_outline',
         'table-github',
         'table-grid',
-        'table-simple_grid',
-        'table-rounded_grid',
-        'table-double_grid',
-        'table-fancy_grid',
-        'table-outline',
-        'table-simple_outline',
-        'table-rounded_outline',
-        'table-double_outline',
-        'table-fancy_outline',
-        'table-pipe',
-        'table-orgtbl',
+        'table-html',
         'table-jira',
+        'table-latex',
+        'table-latex_booktabs',
+        'table-latex_longtable',
+        'table-latex_raw',
+        'table-mediawiki',
+        'table-moinmoin',
+        'table-orgtbl',
+        'table-outline',
+        'table-pipe',
+        'table-plain',
         'table-presto',
         'table-pretty',
         'table-psql',
+        'table-rounded_grid',
+        'table-rounded_outline',
         'table-rst',
-        'table-mediawiki',
-        'table-moinmoin',
-        'table-youtrack',
-        'table-html',
-        'table-unsafehtml',
-        'table-latex',
-        'table-latex_raw',
-        'table-latex_booktabs',
-        'table-latex_longtable',
+        'table-simple',
+        'table-simple_grid',
+        'table-simple_outline',
         'table-textile',
-        'table-tsv'
+        'table-tsv',
+        'table-unsafehtml',
+        'table-youtrack',
+        'yaml',
     ],
-    case_sensitive=False
+    case_sensitive=False,
 )
-
