@@ -1,12 +1,13 @@
 import click
-from ..helpers.build_britive import build_britive
-from ..options.britive_options import britive_options
+
+from pybritive.helpers.build_britive import build_britive
+from pybritive.options.britive_options import britive_options
 
 
 @click.command()
 @build_britive
-@britive_options(names='tenant,passphrase')
-def logout(ctx, tenant, passphrase):
+@britive_options(names='tenant,silent,passphrase')
+def logout(ctx, tenant, silent, passphrase):
     """Logout of an interactive login session.
 
     This only applies when an API token has not been specified via `--token,-T` or via environment variable
