@@ -1,20 +1,26 @@
-import random
 import base64
-import hashlib
-import time
-import webbrowser
-import requests
-from requests.adapters import HTTPAdapter, Retry
-from pathlib import Path
-import click
 import configparser
+import hashlib
 import json
 import os
-from .encryption import StringEncryption, InvalidPassphraseException
-from britive.britive import Britive
-from dateutil import parser
-import jwt
+import random
+import time
+import uuid
+import webbrowser
+from pathlib import Path
+from typing import Optional
 
+import click
+import jwt
+import requests
+from britive.helpers.utils import (
+    parse_tenant,
+    source_federation_token,
+)
+from dateutil import parser
+from requests.adapters import HTTPAdapter, Retry
+
+from .encryption import InvalidPassphraseException, StringEncryption
 
 interactive_login_fields_to_pop = [
     'challengeParameters',
@@ -25,14 +31,10 @@ interactive_login_fields_to_pop = [
     'username',
     'refreshToken',
     'authTime',
-    'maxSessionTimeout'
+    'maxSessionTimeout',
 ]
 
 
-# the credentials should expire sooner than the true expiration date
-# in case we need to do things like polling for credentials during
-# an approval process
-credential_expiration_safe_zone_minutes = 10
 federation_provider_default_expiration_seconds = 900
 
 
@@ -44,49 +46,73 @@ def b64_encode_url_safe(value: bytes):
     return base64.urlsafe_b64encode(value).decode('utf-8').replace('=', '')
 
 
+class CouldNotExtractExpirationTimeFromJwtException(Exception):
+    pass
+
+
 # this base class expects self.credentials to be a dict - so sub classes need to convert to dict
 class CredentialManager:
-    def __init__(self, tenant_name: str, tenant_alias: str, cli: any, federation_provider: str = None):
+    def __init__(
+        self,
+        tenant_name: str,
+        tenant_alias: str,
+        cli: any,
+        federation_provider: Optional[str] = None,
+        browser: str = os.getenv('PYBRITIVE_BROWSER'),
+    ):
         self.cli = cli
         self.tenant = tenant_name
         self.alias = tenant_alias
-        self.base_url = f'https://{Britive.parse_tenant(tenant_name)}'
+        self.base_url = f'https://{parse_tenant(tenant_name)}'
         self.federation_provider = federation_provider
+        self.browser = browser
         self.session = None
 
-        # not sure if we really need 32 random bytes or if any random string would work
-        # but the current britive-cli in node.js does it this way so it will be done the same
-        # way in python
-        self.verifier = b64_encode_url_safe(bytes([random.getrandbits(8) for _ in range(0, 32)]))
-        self.auth_token = b64_encode_url_safe(bytes(hashlib.sha512(self.verifier.encode('utf-8')).digest()))
+        # not sure if we really need 32 random bytes or if any random string would work, but it was carried forward from
+        # the now deprecated node.js CLI, so it remains for the time being.
+        while True:  # will break eventually when we get values that do not include --
+            self.verifier = b64_encode_url_safe(bytes([random.getrandbits(8) for _ in range(0, 32)]))
+            self.auth_token = b64_encode_url_safe(bytes(hashlib.sha512(self.verifier.encode('utf-8')).digest()))
+
+            # WAF doesn't like to see `--` as it thinks it is a sql injection attempt
+            if '--' not in self.verifier and '--' not in self.auth_token:
+                break
+
         self.credentials = self.load() or {}
 
     def _setup_requests_session(self):
         self.session = requests.Session()
         retries = Retry(total=5, backoff_factor=1, status_forcelist=[429, 500, 502, 503, 504])
         self.session.mount('https://', HTTPAdapter(max_retries=retries))
-
+        if global_ca_bundle := self.cli.config.global_ca_bundle:
+            os.environ['PYBRITIVE_CA_BUNDLE'] = global_ca_bundle
+            self.session.verify = global_ca_bundle
         # allow the disabling of TLS/SSL verification for testing in development (mostly local development)
         if os.getenv('BRITIVE_NO_VERIFY_SSL') and '.dev.' in self.tenant:
             # turn off ssl verification
             self.session.verify = False
             # wipe these due to this bug: https://github.com/psf/requests/issues/3829
-            os.environ['CURL_CA_BUNDLE'] = ""
-            os.environ['REQUESTS_CA_BUNDLE'] = ""
+            os.environ['CURL_CA_BUNDLE'] = ''
+            os.environ['REQUESTS_CA_BUNDLE'] = ''
             # disable the warning message
             import urllib3
+
             urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
 
     def perform_interactive_login(self):
         self.cli.print(f'Performing interactive login against tenant {self.tenant}.')
-        url = f'{self.base_url}/login?token={self.auth_token}'
+        sso_idp = self.cli.config.get_tenant().get('sso_idp')
+        if sso_idp:
+            sso_idp = sso_idp.replace('saml', 'SAML')  # ui is expecting it in caps
+            url = f'{self.base_url}/sso?idp={sso_idp}&token={self.auth_token}'
+        else:
+            url = f'{self.base_url}/login?token={self.auth_token}'
 
         # establish a requests session which will be used in retrieve_tokens()
         self._setup_requests_session()
 
         try:
-            webbrowser.get()
-            webbrowser.open(url)
+            webbrowser.get(using=self.browser).open(url)
         except webbrowser.Error:
             self.cli.print('No web browser found. Please manually navigate to the link below and authenticate.')
             self.cli.print(url)
@@ -95,7 +121,7 @@ class CredentialManager:
         num_tries = 1
         while True:
             if num_tries > 60:
-                raise InteractiveLoginTimeout()
+                raise InteractiveLoginTimeout
             response = self.retrieve_tokens()
 
             if response.status_code >= 400:
@@ -103,12 +129,21 @@ class CredentialManager:
                 num_tries += 1
             else:
                 credentials = response.json()['authenticationResult']
+                try:
+                    # attempt to pull the expiration time from the jwt
+                    expiration_time_ms = self._extract_exp_from_jwt(
+                        token=credentials['accessToken'], verify=False, convert_to_ms=True
+                    )
+                    self.cli.debug(f'found expiration time {expiration_time_ms} from jwt')
+                except CouldNotExtractExpirationTimeFromJwtException:
+                    # calculate from other fields in the authentication result
+                    self.cli.debug('could not extract token expiration time from jwt - dropping to use other fields')
+                    auth_time = int(credentials.get('authTime', 0))
+                    session_time = int(credentials.get('maxSessionTimeout', 0))
+                    expiration_time_ms = auth_time + session_time
+                    self.cli.debug(f'found expiration time {expiration_time_ms} from authTime + maxSessionTimeout')
 
-                # calculate a safe expiration time
-                auth_time = int(credentials.get('authTime', 0))
-                session_time = int(credentials.get('maxSessionTimeout', 0))
-                creds_expire_after = auth_time + session_time - (credential_expiration_safe_zone_minutes * 60 * 1000)
-                credentials['safeExpirationTime'] = creds_expire_after
+                credentials['safeExpirationTime'] = expiration_time_ms
 
                 # drop a bunch of unnecessary fields
                 for field in interactive_login_fields_to_pop:
@@ -118,9 +153,39 @@ class CredentialManager:
                 self.cli.print(f'Authenticated to tenant {self.tenant} via interactive login.')
                 break
 
+    @staticmethod
+    def extract_field_from_jwt(token: str, field: str, verify: bool = False):
+        try:
+            return jwt.decode(
+                token,
+                # validation of the token will occur on the Britive backend
+                # so not verifying everything here is okay since we are just
+                # trying to extract the token expiration time so we can store
+                # it in the ~/.britive/pybritive.credentials[.encrypted] file
+                options={'verify_signature': verify, 'verify_aud': verify},
+            )[field]
+        except Exception:
+            return None
+
+    @staticmethod
+    def _extract_exp_from_jwt(token: str, verify: bool = False, convert_to_ms: bool = False):
+        try:
+            expiration_time = jwt.decode(
+                token,
+                # validation of the token will occur on the Britive backend
+                # so not verifying everything here is okay since we are just
+                # trying to extract the token expiration time so we can store
+                # it in the ~/.britive/pybritive.credentials[.encrypted] file
+                options={'verify_signature': verify, 'verify_aud': verify},
+            )['exp']
+            return expiration_time * (1000 if convert_to_ms else 1)
+        except Exception as e:
+            raise CouldNotExtractExpirationTimeFromJwtException from e
+
     def perform_federation_provider_authentication(self):
-        self.cli.print(f'Performing {self.federation_provider} federation provider authentication '
-                       f'against tenant {self.tenant}.')
+        self.cli.print(
+            f'Performing {self.federation_provider} federation provider authentication against tenant {self.tenant}.'
+        )
 
         # we need to extract the duration, if provided
         # field format is provider-[something provider specific]_[duration in seconds]
@@ -130,15 +195,12 @@ class CredentialManager:
             try:
                 duration = int(helper[1])
             except ValueError:
-                self.cli.print(f'Invalid federation provider duration {helper[1]} provided - defaulting '
-                               f'to {duration} seconds.')
+                self.cli.print(
+                    f'Invalid federation provider duration {helper[1]} provided - defaulting to {duration} seconds.'
+                )
 
         # generate the token
-        generated_token = Britive.source_federation_token_from(
-            provider=helper[0],
-            tenant=self.tenant,
-            duration_seconds=duration
-        )
+        generated_token = source_federation_token(provider=helper[0], tenant=self.tenant, duration_seconds=duration)
 
         # obtain the provider and expiration time of the token
         provider, token = generated_token.split('::')
@@ -153,52 +215,35 @@ class CredentialManager:
                 token_expires = json.loads(token)['iam_request_headers']['x-britive-expires']
                 expiration_time = int(parser.parse(token_expires).timestamp() * 1000)
             if provider == 'oidc':
-                expiration_time = jwt.decode(
-                    token,
-                    # validation of the token will occur on the Britive backend
-                    # so not verifying everything here is okay since we are just
-                    # trying to extract the token expiration time so we can store
-                    # it in the ~/.britive/pybritive.credentials[.encrypted] file
-                    options={
-                        'verify_signature': False,
-                        'verify_aud': False
-                    }
-                )['exp'] * 1000
-        except Exception as e:
-            self.cli.print(f'Cannot obtain token expiration time for {self.federation_provider}. Defaulting to '
-                           f'{federation_provider_default_expiration_seconds} seconds.')
+                expiration_time = self._extract_exp_from_jwt(token=token, verify=False, convert_to_ms=True)
+        except Exception:
+            self.cli.print(
+                f'Cannot obtain token expiration time for {self.federation_provider}. Defaulting to '
+                f'{federation_provider_default_expiration_seconds} seconds.'
+            )
 
         # generate the credentials object and save it
-        credentials = {
-            'accessToken': generated_token,
-            'safeExpirationTime': expiration_time
-        }
+        credentials = {'accessToken': generated_token, 'safeExpirationTime': expiration_time}
 
         self.save(credentials)
         self.cli.print(f'Authenticated to tenant {self.tenant} via federation provider authentication.')
 
     def retrieve_tokens(self):
         url = f'{self.base_url}/api/auth/cli/retrieve-tokens'
-        auth_params = {
-            'authParameters': {
-                'cliToken': self.verifier
-            }
-        }
-        headers = {
-            'Content-Type': 'application/json'
-        }
+        auth_params = {'authParameters': {'cliToken': self.verifier}}
+        headers = {'Content-Type': 'application/json'}
         return self.session.post(url, headers=headers, json=auth_params)
 
     def load(self, full=False):
-        # we should NEVER get herte but adding here just in case
+        # we should NEVER get here but adding here just in case
         raise click.ClickException('Must use a subclass of CredentialManager')
 
     def save(self, credentials: dict):
-        # we should NEVER get herte but adding here just in case
+        # we should NEVER get here but adding here just in case
         raise click.ClickException('Must use a subclass of CredentialManager')
 
     def delete(self):
-        # we should NEVER get herte but adding here just in case
+        # we should NEVER get here but adding here just in case
         raise click.ClickException('Must use a subclass of CredentialManager')
 
     # this helper exists since subclasses may need to override the method due to how the
@@ -208,7 +253,7 @@ class CredentialManager:
 
     def get_token(self):
         if not self.has_valid_credentials():  # no credentials or expired creds for the tenant so do interactive login
-
+            self.cli.debug('has_valid_credentials = False')
             # both methods below write the credentials out and update self.credentials as needed
             if self.federation_provider:
                 self.perform_federation_provider_authentication()
@@ -222,22 +267,30 @@ class CredentialManager:
             self.cli.print(f'Credentials for tenant {self.tenant} not found.')
             return False
         if int(time.time() * 1000) <= int(self.credentials.get('safeExpirationTime', 0)):
+            self.cli.debug('credentials.py::has_valid_credentials - credentials exist and are not expired so are valid')
             return True
         self.cli.print(f'Credentials for tenant {self.tenant} have expired.')
         return False
 
 
 class FileCredentialManager(CredentialManager):
-    def __init__(self, tenant_name: str, tenant_alias: str, cli: any, federation_provider: str = None):
+    def __init__(
+        self,
+        tenant_name: str,
+        tenant_alias: str,
+        cli: any,
+        federation_provider: Optional[str] = None,
+        browser: str = os.getenv('PYBRITIVE_BROWSER'),
+    ):
         home = os.getenv('PYBRITIVE_HOME_DIR', str(Path.home()))
         self.path = str(Path(home) / '.britive' / 'pybritive.credentials')
-        super().__init__(tenant_name, tenant_alias, cli, federation_provider)
+        super().__init__(tenant_name, tenant_alias, cli, federation_provider, browser)
 
     def load(self, full=False):
         path = Path(self.path)
         if not path.is_file():  # credentials file does not yet exist, create it as an empty file
             path.parent.mkdir(exist_ok=True, parents=True)
-            path.write_text('')
+            path.write_text('', encoding='utf-8')
 
         # open the file with configparser
         credentials = configparser.ConfigParser()
@@ -252,39 +305,56 @@ class FileCredentialManager(CredentialManager):
         full_credentials = self.load(full=True)
         if credentials is None:
             full_credentials.pop(self.alias, None)
+            self.credentials = None
         else:
             full_credentials[self.alias] = credentials
+            # effectively a deep copy
+            self.credentials = json.loads(json.dumps(credentials))
 
         config = configparser.ConfigParser()
         config.optionxform = str  # maintain key case
         config.read_dict(full_credentials)
 
         # write the new credentials file
-        with open(str(self.path), 'w') as f:
+        with open(str(self.path), 'w', encoding='utf-8') as f:
             config.write(f, space_around_delimiters=False)
-        self.credentials = credentials
+
+        jti = self.extract_field_from_jwt(token=(self.credentials or {}).get('accessToken'), verify=False, field='jti')
+        self.cli.debug(f'credentials.py::FileCredentialManager::save - set credentials to jwt id {jti}')
 
     def delete(self):
         self.save(None)
 
 
 class EncryptedFileCredentialManager(CredentialManager):
-    def __init__(self, tenant_name: str, tenant_alias: str, cli: any, passphrase: str = None,
-                 federation_provider: str = None):
+    def __init__(
+        self,
+        tenant_name: str,
+        tenant_alias: str,
+        cli: any,
+        passphrase: Optional[str] = None,
+        federation_provider: Optional[str] = None,
+        browser: str = os.getenv('PYBRITIVE_BROWSER'),
+    ):
         home = os.getenv('PYBRITIVE_HOME_DIR', str(Path.home()))
         self.path = str(Path(home) / '.britive' / 'pybritive.credentials.encrypted')
         self.passphrase = passphrase
         self.string_encryptor = StringEncryption(passphrase=self.passphrase)
-        super().__init__(tenant_name, tenant_alias, cli, federation_provider)
+        super().__init__(tenant_name, tenant_alias, cli, federation_provider, browser)
 
     def decrypt(self, encrypted_access_token: str):
         try:
             return self.string_encryptor.decrypt(ciphertext=encrypted_access_token)
         except InvalidPassphraseException:
-            self.cli.print('invalid passphrase provided - wiping credentials and forcing a re-authentication.')
-            self.delete()
-            self.credentials = self.load() or {}
-            return self.get_token()
+            try:
+                self.passphrase = str(uuid.getnode())
+                self.string_encryptor = StringEncryption(passphrase=self.passphrase)
+                return self.string_encryptor.decrypt(ciphertext=encrypted_access_token)
+            except InvalidPassphraseException:
+                self.cli.print('invalid passphrase provided - wiping credentials and forcing a re-authentication.')
+                self.delete()
+                self.credentials = self.load() or {}
+                return self.get_token()
 
     def encrypt(self, decrypted_access_token: str):
         return self.string_encryptor.encrypt(plaintext=decrypted_access_token)
@@ -293,7 +363,7 @@ class EncryptedFileCredentialManager(CredentialManager):
         path = Path(self.path)
         if not path.is_file():  # credentials file does not yet exist, create it as an empty file
             path.parent.mkdir(exist_ok=True, parents=True)
-            path.write_text('')
+            path.write_text('', encoding='utf-8')
 
         # open the file with configparser
         credentials = configparser.ConfigParser()
@@ -313,6 +383,7 @@ class EncryptedFileCredentialManager(CredentialManager):
         full_credentials = self.load(full=True)
         if credentials is None:
             full_credentials.pop(self.alias, None)
+            self.credentials = None
         else:
             credentials['accessToken'] = self.encrypt(credentials['accessToken'])
             full_credentials[self.alias] = credentials
@@ -324,8 +395,11 @@ class EncryptedFileCredentialManager(CredentialManager):
         config.read_dict(full_credentials)
 
         # write the new credentials file
-        with open(str(self.path), 'w') as f:
+        with open(str(self.path), 'w', encoding='utf-8') as f:
             config.write(f, space_around_delimiters=False)
 
+        jti = self.extract_field_from_jwt(token=(self.credentials or {}).get('accessToken'), verify=False, field='jti')
+        self.cli.debug(f'credentials.py::FileCredentialManager::save - set credentials to jwt id {jti}')
+
     def delete(self):
         self.save(None)
