@@ -1,11 +1,14 @@
-import uuid
+import base64
+import hashlib
+import os
+import platform
+from getpass import getuser
+from typing import Optional
+
 from cryptography.fernet import Fernet, InvalidToken
 from cryptography.hazmat.backends import default_backend
 from cryptography.hazmat.primitives import hashes
 from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
-import os
-import base64
-import click
 
 
 class InvalidPassphraseException(Exception):
@@ -13,8 +16,13 @@ class InvalidPassphraseException(Exception):
 
 
 class StringEncryption:
-    def __init__(self, passphrase: str = None):
-        self.passphrase = passphrase or str(uuid.getnode())  # TODO change?
+    def __init__(self, passphrase: Optional[str] = None):
+        self.passphrase = (
+            passphrase
+            or hashlib.sha256(
+                '|'.join([getuser(), *platform.uname()._asdict().values()]).replace(' ', '').encode('utf-8')
+            ).hexdigest()
+        )
 
     @staticmethod
     def _salt():
@@ -26,7 +34,7 @@ class StringEncryption:
             length=32,
             salt=base64.b64decode(salt.encode()),
             iterations=100000,
-            backend=default_backend()
+            backend=default_backend(),
         )
         return base64.urlsafe_b64encode(kdf.derive(self.passphrase.encode()))
 
@@ -41,5 +49,5 @@ class StringEncryption:
             ciphertext, b64salt = ciphertext.split(':')
             key = self._key(b64salt)
             return Fernet(key).decrypt(base64.b64decode(ciphertext.encode())).decode('utf-8')
-        except InvalidToken:
-            raise InvalidPassphraseException()
+        except InvalidToken as e:
+            raise InvalidPassphraseException from e
