@@ -1,7 +1,7 @@
 def profile_split(profile):
     def str_escape_split(str_to_escape, delimiter=',', escape='\\'):
         if len(delimiter) > 1 or len(escape) > 1:
-            raise ValueError("Either delimiter or escape must be an one char value")
+            raise ValueError('Either delimiter or escape must be an one char value')
         token = ''
         escaped = False
         for c in str_to_escape:
@@ -25,4 +25,5 @@ def profile_split(profile):
                     escaped = False
                 token += c
         yield token
+
     return list(str_escape_split(profile, delimiter='/', escape='\\'))
