@@ -1,12 +1,11 @@
 import click
 
-
 option = click.option(
     '--aws-credentials-file',
     default=None,
     help='AWS Programmatic Only - When mode is `integrate` specify a non-default location for the AWS '
-         'credentials file.',
+    'credentials file.',
     envvar='AWS_SHARED_CREDENTIALS_FILE',
     show_envvar=True,
-    show_default=True
+    show_default=True,
 )
