@@ -1,11 +1,10 @@
 import click
 
-
 option = click.option(
-    '--checked-out', '-c',
+    '--checked-out',
+    '-c',
     default=False,
     is_flag=True,
     show_default=True,
-    help='Filter profile list to currently checked out profiles.'
+    help='Filter profile list to currently checked out profiles.',
 )
-
