@@ -1,21 +1,23 @@
-import click
 import platform
-import pkg_resources
+from importlib.metadata import version
+
+import click
 
 
 # for --version we need to print the version and exit - nothing more, nothing less
 def version_callback(ctx, self, value):
     if value:
-        cli_version = pkg_resources.get_distribution('pybritive').version
+        cli_version = version('pybritive')
         click.echo(f'pybritive: {cli_version} / platform: {platform.platform()} / python: {platform.python_version()}')
-        raise click.exceptions.Exit()
+        raise click.exceptions.Exit
 
 
 option = click.option(
-    '--version', '-v',
+    '--version',
+    '-v',
     default=None,
     callback=version_callback,
     is_eager=True,
     help='Prints the PyBritive CLI version and exits.',
-    is_flag=True
+    is_flag=True,
 )
